
get_zero_obs<- function(x,date) {
  timeSeries(matrix(0,1,ncol(x)),as.Date(date),colnames(x))
}



#' Update quantities and prices of an object of class "myptf"
#' 
#' Update time series of prices and quantities in a \code{\link{myptf}}
#' by adding new observations.
#'
#' @param x object of class \code{\link{myptf}}.
#' @param newquantities,newprices either a \code{timeSeries} object or an object
#'   coercible to \code{timeSeries} containing new information on quantities or
#'   prices.
#'
#' @return object of classs \code{\link{myptf}} with updated information.
#'
#' @seealso \code{\link{myptf}}
#'
#' @export
update_quantities<- function(x,newquantities) {
  if (!inherits(x,'myptf')) {
    stop('Object of class "myptf" required for argument "x"')
  }
  
  depo<- colnames(x$quantities)
  x$quantities<- rbind(x$quantities,as.timeSeries(newquantities))
  colnames(x$quantities)<- depo
  x$quantities<- sort(x$quantities)
  x
}



#' @rdname update_quantities
#' @export
update_prices<- function(x,newprices) {
  if (!inherits(x,'myptf')) {
    stop('Object of class "myptf" required for argument "x"')
  }
  
  depo<- colnames(x$prices)
  x$quantities<- rbind(x$prices,as.timeSeries(newprices))
  colnames(x$prices)<- depo
  x$prices<- sort(x$prices)
  x
}



#' Extract quantities and prices from an object of class "myptf"
#' 
#' \code{get_quantities} and \code{get_prices} extract quantities and prices
#' as a \code{timeSeries} object at the specified date.
#'
#' @param x object of class \code{\link{myptf}}.
#' @param date either a \code{Date} object or an object coercible
#'   to \code{Date}.
#'
#' @return an object of classs \code{timeSeries}.
#'
#' @family value functions
#'
#' @export
get_quantities<- function(x,date=Sys.Date()) {
  if (!inherits(x,'myptf')) {
    stop('Object of class "myptf" required for argument "x"')
  }
  
  out<- window(x$quantities,start(x$quantities),as.Date(date))

  if (nrow(out)==0) { out<- get_zero_obs(out,date) }

  out<- sort(out,decreasing=TRUE)
  
  out[1,]
}



#' @rdname get_quantities
#' @export
get_prices<- function(x,date=Sys.Date()) {
  if (!inherits(x,'myptf')) {
    stop('Object of class "myptf" required for argument "x"')
  }
  
  out<- window(x$prices,start(x$prices),as.Date(date))

  if (nrow(out)==0) { out<- get_zero_obs(out,date) }

  out<- sort(out,decreasing=TRUE)
  
  out[1,]
}



#' Compute the value of a portfolio
#' 
#' Compute the value of a \code{\link{myptf}} portfolio at a given date
#' (\code{get_value}) or at each date when prices are aviailable since
#' the creation of the portfolio (\code{get_hist_value}).
#'
#' @param x object of class \code{\link{myptf}}.
#' @param date either a \code{Date} object or an object coercible
#'   to \code{Date}.
#'
#' @return numeric vector.
#'
#' @family value functions
#'
#' @export
get_value<- function(x,date=Sys.Date()) {
  sum(as.numeric(get_quantities(x,date))*as.numeric(get_prices(x,date)))
}


#' @rdname get_value
#' @export
get_hist_value<- function(x) {
  date<- as.Date(rownames(window(x$prices,start(x$quantities),Sys.Date())))
  values<- sapply(date,get_value,x=x)
  timeSeries(values,date,x$code)
}



#' Compute the composition a portfolio
#' 
#' Compute either value composition or relative composition of a 
#' \code{\link{myptf}} portfolio at a given date.
#'
#' @param x object of class \code{\link{myptf}}.
#' @param date either a \code{Date} object or an object coercible
#'   to \code{Date}.
#' @param value if \code{TRUE} it returns the composition in value,
#'   if \code{FALSE} (default) the relative composition is computed.
#'
#' @return numeric vector.
#'
#' @family value functions
#'
#' @export
get_composition<- function(x,date=Sys.Date(),value=FALSE) {
  if (!inherits(x,'myptf')) {
    stop('Object of class "myptf" required for argument "x"')
  }
  out<- as.numeric(get_quantities(x,date))*as.numeric(get_prices(x,date))
  if (!value) { out<- out/sum(out) }
  names(out)<- colnames(x$prices)
  out
}



